import hashlib
from django.http import HttpResponse
from django.core.validators import validate_email
from django.core.exceptions import ValidationError
from main import default_vm_node, default_iso_storage, default_storage
from main import proxmoxAuth
from main import VmProxMoxDTO
from main.db_functions import get_default_settings, is_iso_in_sql_db, insert_iso, get_list_iso_from_db, remove_iso, \
    insert_member
from main import UserProxMoxDTO
from datetime import datetime, timedelta
import re
from main.models import Iso


def create_vm(vmdto: VmProxMoxDTO, vm_id: int):
    # Get parameters for VM
    node = vmdto.node  # Node name
    vm_name = re.sub('[^a-zA-Z0-9]', '', vmdto.vm_name)  # Name for VM sanitized of prohibited symbols
    vm_cores = vmdto.vm_cores  # Number of CPU cores
    vm_memory = vmdto.vm_memory  # Memory size in MB
    vm_storage = vmdto.vm_storage  # Storage ID where the VM will be created
    vm_disk_size = vmdto.vm_disk_size  # Disk size for the VM
    iso_file = vmdto.vm_iso_file  # Path to the ISO file

    config = {
        'name': vm_name,
        'cores': vm_cores,
        'memory': vm_memory,
        'storage': vm_storage,
        'ide2': f'{vm_storage}:iso/proxmox-ve_*.iso,size={vm_disk_size}',  # Disk size
        'boot': 'cdn',
        'cdrom': iso_file,
    }
    # Send request to create a VM
    out = proxmoxAuth.nodes(node).qemu.create(vmid=vm_id, **config)
    return out


# ('pve', 'local')
def get_list_iso_urls(node, storage) -> list:
    # Request list of all ISO files
    iso_info = proxmoxAuth.nodes(node).storage(storage).content.get(content='iso')

    # Create a dictionary of iso_name - iso_path
    iso_list_output = []
    for record in iso_info:
        iso_list_output.append(record['volid'])

    return iso_list_output


def get_list_templates() -> list:
    template_info = proxmoxAuth.cluster.resources.get(type='vm')
    templates = []
    for record in template_info:
        try:
            if record['template'] == 1:
                templates.append(record)
        except KeyError:
            continue # If vm not fully initialized -> skip it
    return templates


def get_list_all_users() -> list:
    user_info = proxmoxAuth.content.get(content="username")

    user_list_output = []
    for user in user_info:
        user_list_output.append(user['email'])

    return user_list_output


def months_to_days(num_months):
    current_date = datetime.now()
    future_date = current_date + timedelta(days=(num_months * 365) / 12)
    days_difference = (future_date - current_date).days
    return days_difference


def sanitize_vm_data_input(vminp: VmProxMoxDTO) -> VmProxMoxDTO:
    # Get a list of limits from mysql DB
    limits = get_default_settings()
    # Validate input against limits and change data accordingly
    if limits['storage_gbs'] < int(vminp.vm_disk_size[:-1]):  # Remove "G" at the end of disk size
        vminp.vm_disk_size = limits['storage_gbs']
    if limits['memory_mbs'] < vminp.vm_memory:
        vminp.vm_memory = limits['memory_mbs']
    if limits['cores'] < vminp.vm_cores:
        vminp.vm_cores = limits['cores']
    if months_to_days(int(limits['lease_time_months'])) < vminp.vm_expire_days:
        vminp.vm_expire_days = months_to_days(limits['lease_time_months'])
    return vminp


def sync_iso_from_ve_to_db():
    proxmox_isos = get_list_iso_urls(default_vm_node, default_iso_storage)
    for volid in proxmox_isos:
        if not is_iso_in_sql_db(volid):
            # If iso is not in sql then save its data
            insert_iso(volid, volid)
    # If sql has ISO that no longer exists in VE, remove it
    sql_iso = get_list_iso_from_db()
    for record in sql_iso:
        if record.volid not in proxmox_isos:
            remove_iso(record.id)


# Returns volid of the uploaded file
def upload_iso(iso_file, upload_path) -> str:
    # Upload to Proxmox VE
    proxmoxAuth.nodes(default_vm_node).storage(default_iso_storage).upload.create(filename=open(upload_path, 'rb'),
                                                                                  content='iso')
    # Get last uploaded ISO's info for volid
    iso_files = proxmoxAuth.nodes('pve').storage('local').content.get()
    last_uploaded_iso = max(iso_files, key=lambda x: x['ctime'])
    return last_uploaded_iso['volid']


def remove_iso_from_ve(iso_id: str) -> HttpResponse:
    try:
        iso_model = Iso.objects.get(id=iso_id)
    except Iso.DoesNotExist:
        return HttpResponse("Object not found.")

    try:
        # If the object in db is found -> remove
        proxmoxAuth.nodes(default_vm_node).storage(default_iso_storage).content.delete(iso_model.volid)
        return HttpResponse("File removed!")
    except Exception as e:
        return HttpResponse({str(e)})


def create_user(user: UserProxMoxDTO, useridinp: int):
    config = {
        'password': user.password,
        'firstname': user.firstname,
        'lastname': user.lastname,
        'email': user.email,
        'enable': user.enable,
    }
    user_name_realm = str(user.username) + '@' + "pve"
    out = proxmoxAuth.access.users.post(userid=user_name_realm, **config)


def sha256_hash_string(input: str) -> str:
    # Convert the input to bytes
    password_bytes = input.encode('utf-8')
    # Generate the SHA-256 hash
    sha256_hash = hashlib.sha256(password_bytes).hexdigest()
    return sha256_hash


def is_email_nhl(email: str) -> bool:
    try:
        validate_email(email)
        domain = email.split("@")[1]
        if domain == "nhlstenden.com" or domain == "student.nhlstenden.com":
            return True
        else:
            return False
    except ValidationError:
        return False


def change_user_enable(username: str) -> HttpResponse:
    username_formatted = username + '@pve'
    user = proxmoxAuth.access.users(username_formatted).put(enable=1)
    return user

# def sync_users_ve_db() -> HttpResponse:
#     proxmox_user = get_list_all_users()
#     for user in proxmox_user:
#         if not is_iso_in_sql_db(user):
#             # If iso is not in sql then save its data
#             insert_member(user, user)
#         # If sql has ISO that no longer exists in VE, remove it
#     sql_iso = get_list_iso_from_db()
#     for record in sql_iso:
#         if record.volid not in proxmox_isos:
#             remove_iso(record.id)


def create_vm_from_template(vmid: int, vmname: str, templateid: int) -> HttpResponse:
    response = proxmoxAuth.nodes(default_vm_node).qemu(templateid).clone.create(newid=vmid, name=vmname, full=1)
    return response


# Check if a logged in user is a superuser
def superuser_check(user):
    return user.is_superuser
